def len2(x):
	if x.__class__ is str:
		return len([x])
	else:
		return len(x)

def check_data(x,y):
	if len2(x) == 1:
		testdata = [x]
	else:
		testdata = x

	for z in testdata:
		if z not in y:
			raise TypeError(z + ' is not one of the choices')

def get_meta(x):
  if has_meta(x):
  	return { z: x[z] for z in ['offset','limit','endOfRecords'] }
  else:
  	return None

def has_meta(x):
	if x.__class__ != dict:
		return False
	else:
		tmp = [y in x.keys() for y in ['offset','limit','endOfRecords']]
		return True in tmp

def parse_results(x, y):
  if y.__class__.__name__ != 'NoneType':
  	if y.__class__ != dict:
  		return x
  	else:
	    if 'endOfRecords' in x.keys():
	      return x['results']
	    else:
	      return x
  else:
    return x['results']



"""Tests for occurrences module - get methods"""
import os
from pygbif import occurrences

def test_get():
    "occurrences.get - basic test"
    res = occurrences.get(key = 252408386)
    assert 'dict' == res.__class__.__name__
    assert 30 == len(res)
    assert 252408386 == res['key']

def test_get_verbatim():
    "occurrences.get_verbatim - basic test"
    res = occurrences.get_verbatim(key = 252408386)
    assert 'dict' == res.__class__.__name__
    assert 24 == len(res)
    assert 252408386 == res['key']

def test_get_fragment():
    "occurrences.get_fragment - basic test"
    res = occurrences.get_fragment(key = 1052909293)
    assert 'dict' == res.__class__.__name__
    assert 41 == len(res)
    assert 'HumanObservation' == res['basisOfRecord']



"""Tests for species module - name_backbone methods"""
import os
from pygbif import species

def test_name_backbone():
    "species.name_backbone - basic test"
    res = species.name_backbone(name='Helianthus annuus')
    assert dict == res.__class__
    assert 22 == len(res)
    assert 'Helianthus annuus' == res['species']

def test_name_backbone_multiple_matches():
    "species.name_backbone - multiple matches"
    res = species.name_backbone(name='Aso')
    assert dict == res.__class__
    assert 4 == len(res)
    assert 'Multiple equal matches for Aso' == res['note']



# -*- coding: utf-8 -*-

from .search import search
from .get import get, get_verbatim, get_fragment
from .count import count, count_basisofrecord, count_year, \
    count_datasets, count_countries, count_schema, count_publishingcountries



from ..gbifutils import *
from .registry_utils import *

def nodes(data = 'all', uuid = None, query = None, identifier = None,
  identifierType = None, limit = 100, start = None, isocode = None, **kwargs):
  '''
  Nodes metadata.

  :param data: The type of data to get. Default is all data.
  :param uuid: UUID of the data node provider. This must be specified if data
     is anything other than 'all'.
  :param query: Query nodes. Only used when data='all'
  :param isocode: A 2 letter country code. Only used if data='country'.

  References http://www.gbif.org/developer/registry#nodes

  Usage::

      from pygbif import registry
      registry.nodes(limit=5)
      registry.nodes(uuid="1193638d-32d1-43f0-a855-8727c94299d8")
      registry.nodes(data='identifier', uuid="03e816b3-8f58-49ae-bc12-4e18b358d6d9")
      registry.nodes(data=['identifier','organization','comment'], uuid="03e816b3-8f58-49ae-bc12-4e18b358d6d9")

      uuids = ["8cb55387-7802-40e8-86d6-d357a583c596","02c40d2a-1cba-4633-90b7-e36e5e97aba8",
      "7a17efec-0a6a-424c-b743-f715852c3c1f","b797ce0f-47e6-4231-b048-6b62ca3b0f55",
      "1193638d-32d1-43f0-a855-8727c94299d8","d3499f89-5bc0-4454-8cdb-60bead228a6d",
      "cdc9736d-5ff7-4ece-9959-3c744360cdb3","a8b16421-d80b-4ef3-8f22-098b01a89255",
      "8df8d012-8e64-4c8a-886e-521a3bdfa623","b35cf8f1-748d-467a-adca-4f9170f20a4e",
      "03e816b3-8f58-49ae-bc12-4e18b358d6d9","073d1223-70b1-4433-bb21-dd70afe3053b",
      "07dfe2f9-5116-4922-9a8a-3e0912276a72","086f5148-c0a8-469b-84cc-cce5342f9242",
      "0909d601-bda2-42df-9e63-a6d51847ebce","0e0181bf-9c78-4676-bdc3-54765e661bb8",
      "109aea14-c252-4a85-96e2-f5f4d5d088f4","169eb292-376b-4cc6-8e31-9c2c432de0ad",
      "1e789bc9-79fc-4e60-a49e-89dfc45a7188","1f94b3ca-9345-4d65-afe2-4bace93aa0fe"]

      [ registry.nodes(data='identifier', uuid=x) for x in uuids ]
  '''
  args = {'q': query, 'limit': limit, 'offset': start}
  data_choices = ['all', 'organization', 'endpoint',
     'identifier', 'tag', 'machineTag', 'comment',
     'pendingEndorsement', 'country', 'dataset', 'installation']
  check_data(data, data_choices)

  def getdata(x, uuid, args, **kwargs):
    if x is not 'all' and uuid is None:
      stop('You must specify a uuid if data does not equal "all"')

    if uuid is None:
      if x is 'all':
        url = gbif_baseurl + 'node'
      else:
        if isocode is not None and x is 'country':
          url = gbif_baseurl + 'node/country/' + isocode
        else:
          url = gbif_baseurl + 'node/' + x
    else:
      if x is 'all':
        url = gbif_baseurl + 'node/' + uuid
      else:
        url = gbif_baseurl + 'node/' + uuid + '/' + x

    res = gbif_GET(url, args, **kwargs)
    return {'meta': get_meta(res), 'data': parse_results(res, uuid)}

  # Get data
  if len2(data) == 1:
    return getdata(data, uuid, args, **kwargs)
  else:
    return [getdata(x, uuid, args, **kwargs) for x in data]



from ..gbifutils import *
from .registry_utils import *

def networks(data = 'all', uuid = None, query = None, identifier = None,
  identifierType = None, limit = 100, start = None, **kwargs):
  '''
  Networks metadata.

  :param data: The type of data to get. Default is all data.
  :param uuid: UUID of the data network provider. This must be specified if data
     is anything other than 'all'.
  :param query: Query networks. Only used when data='all'. Ignored otherwise.

  References: http://www.gbif.org/developer/registry#networks

  :return: A dict

  Usage::

      from pygbif import registry
      registry.networks(limit=5)
      registry.networks(uuid='16ab5405-6c94-4189-ac71-16ca3b753df7')
      registry.networks(data='endpoint', uuid='16ab5405-6c94-4189-ac71-16ca3b753df7')
  '''
  args = {'q': query, 'limit': limit, 'offset': start}
  data_choices = ['all', 'contact', 'endpoint', 'identifier',
    'tag', 'machineTag', 'comment', 'constituents']
  check_data(data, data_choices)

  def getdata(x, uuid, args, **kwargs):
    if x is not 'all' and uuid is None:
      stop('You must specify a uuid if data does not equal "all"')

    if uuid is None:
      url = gbif_baseurl + 'network'
    else:
      if x is 'all':
        url = gbif_baseurl + 'network/' + uuid
      else:
        url = gbif_baseurl + 'network/' + uuid + '/' + x

    res = gbif_GET(url, args, **kwargs)
    return {'meta': get_meta(res), 'data': parse_results(res, uuid)}

  if len2(data) == 1:
    return getdata(data, uuid, args, **kwargs)
  else:
    return [getdata(x, uuid, args, **kwargs) for x in data]




from ..gbifutils import *

def search(taxonKey=None, scientificName=None, country=None,
    publishingCountry=None, hasCoordinate=None, typeStatus=None,
    recordNumber=None, lastInterpreted=None, continent=None,
    geometry=None, recordedBy=None, basisOfRecord=None, datasetKey=None,
    eventDate=None, catalogNumber=None, year=None, month=None,
    decimalLatitude=None, decimalLongitude=None, elevation=None,
    depth=None, institutionCode=None, collectionCode=None,
    hasGeospatialIssue=None, issue=None, q=None, mediatype=None,
    limit=500, offset=0, **kwargs):
    '''
    Search GBIF occurrences

    :param taxonKey: [Fixnum] A GBIF occurrence identifier
    :param scientificName: A scientific name from the GBIF backbone. All included and synonym taxa are included in the search.
    :param datasetKey: The occurrence dataset key (a uuid)
    :param catalogNumber: An identifier of any form assigned by the source within a physical collection or digital dataset for the record which may not unique, but should be fairly unique in combination with the institution and collection code.
    :param recordedBy: The person who recorded the occurrence.
    :param collectionCode: An identifier of any form assigned by the source to identify the physical collection or digital dataset uniquely within the text of an institution.
    :param institutionCode: An identifier of any form assigned by the source to identify the institution the record belongs to. Not guaranteed to be que.
    :param country: The 2-letter country code (as per ISO-3166-1) of the country in which the occurrence was recorded. See here http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2}
    :param basisOfRecord: Basis of record, as defined in our BasisOfRecord enum here http://gbif.github.io/gbif-api/apidocs/org/gbif/api/vocabulary/BasisOfRecord.html Acceptable values are:

     - FOSSIL_SPECIMEN An occurrence record describing a fossilized specimen.
     - HUMAN_OBSERVATION An occurrence record describing an observation made by one or more people.
     - LITERATURE An occurrence record based on literature alone.
     - LIVING_SPECIMEN An occurrence record describing a living specimen, e.g.
     - MACHINE_OBSERVATION An occurrence record describing an observation made by a machine.
     - OBSERVATION An occurrence record describing an observation.
     - PRESERVED_SPECIMEN An occurrence record describing a preserved specimen.
     - UNKNOWN Unknown basis for the record.

    :param eventDate: Occurrence date in ISO 8601 format: yyyy, yyyy-MM, yyyy-MM-dd, or
       MM-dd. Supports range queries, smaller,larger (e.g., '1990,1991', whereas '1991,1990'
       wouldn't work)
    :param year: The 4 digit year. A year of 98 will be interpreted as AD 98. Supports range queries,
       smaller,larger (e.g., '1990,1991', whereas '1991,1990' wouldn't work)
    :param month: The month of the year, starting with 1 for January. Supports range queries,
       smaller,larger (e.g., '1,2', whereas '2,1' wouldn't work)
    :param q: Query terms. The value for this parameter can be a simple word or a phrase.
    :param decimalLatitude: Latitude in decimals between -90 and 90 based on WGS 84.
       Supports range queries, smaller,larger (e.g., '25,30', whereas '30,25' wouldn't work)
    :param decimalLongitude: Longitude in decimals between -180 and 180 based on WGS 84.
       Supports range queries (e.g., '-0.4,-0.2', whereas '-0.2,-0.4' wouldn't work).
    :param publishingCountry: The 2-letter country code (as per ISO-3166-1) of the
       country in which the occurrence was recorded.
    :param elevation: Elevation in meters above sea level. Supports range queries, smaller,larger
       (e.g., '5,30', whereas '30,5' wouldn't work)
    :param depth: Depth in meters relative to elevation. For example 10 meters below a
       lake surface with given elevation. Supports range queries, smaller,larger (e.g., '5,30',
       whereas '30,5' wouldn't work)
    :param geometry: Searches for occurrences inside a polygon described in Well Known
       Text (WKT) format. A WKT shape written as either POINT, LINESTRING, LINEARRING
       or POLYGON. Example of a polygon: ((30.1 10.1, 20, 20 40, 40 40, 30.1 10.1)) would be queried as http://bit.ly/1BzNwDq}.
    :param hasGeospatialIssue: (logical) Includes/excludes occurrence records which contain spatial
       issues (as determined in our record interpretation), i.e. \code{hasGeospatialIssue=TRUE}
       returns only those records with spatial issues while \code{hasGeospatialIssue=FALSE} includes
       only records without spatial issues. The absence of this parameter returns any
       record with or without spatial issues.
    :param issue: (character) One or more of many possible issues with each occurrence record. See
       Details. Issues passed to this parameter filter results by the issue.
    :param hasCoordinate: (logical) Return only occurence records with lat/long data (TRUE) or
       all records (FALSE, default).
    :param typeStatus: Type status of the specimen. One of many options. See ?typestatus
    :param recordNumber: Number recorded by collector of the data, different from GBIF record
       number. See http://rs.tdwg.org/dwc/terms/#recordNumber} for more info
    :param lastInterpreted: Date the record was last modified in GBIF, in ISO 8601 format:
       yyyy, yyyy-MM, yyyy-MM-dd, or MM-dd.  Supports range queries, smaller,larger (e.g.,
       '1990,1991', whereas '1991,1990' wouldn't work)
    :param continent: Continent. One of africa, antarctica, asia, europe, north_america
       (North America includes the Caribbean and reachies down and includes Panama), oceania,
       or south_america
    :param fields: (character) Default ('all') returns all fields. 'minimal' returns just taxon name,
       key, latitude, and longitute. Or specify each field you want returned by name, e.g.
       fields = c('name','latitude','elevation').
    :param return: One of data, hier, meta, or all. If data, a data.frame with the
       data. hier returns the classifications in a list for each record. meta
       returns the metadata for the entire call. all gives all data back in a list.
    :param mediatype: Media type. Default is NULL, so no filtering on mediatype. Options:
       NULL, 'MovingImage', 'Sound', and 'StillImage'.``
    :param limit: [Fixnum] Number of results to return.
    :param offset: [Fixnum] Start at record X

    :return: A dictionary, of results

    Usage::

        from pygbif import occurrences
        occurrences.search(taxonKey = 3329049)

        # Return 2 results, this is the default by the way
        occurrences.search(taxonKey=3329049, limit=2)

        # Instead of getting a taxon key first, you can search for a name directly
        ## However, note that using this approach (with \code{scientificName="..."})
        ## you are getting synonyms too. The results for using \code{scientifcName} and
        ## \code{taxonKey} parameters are the same in this case, but I wouldn't be surprised if for some
        ## names they return different results
        occurrences.search(scientificName = 'Ursus americanus')
        from pygbif import species
        key = species.name_backbone(name = 'Ursus americanus', rank='species')['usageKey']
        occurrences.search(taxonKey = key)

        # Search by dataset key
        occurrences.search(datasetKey='7b5d6a48-f762-11e1-a439-00145eb45e9a', limit=20)

        # Search by catalog number
        occurrences.search(catalogNumber="49366", limit=20)
        # occurrences.search(catalogNumber=["49366","Bird.27847588"], limit=20)

        # Use paging parameters (limit and start) to page. Note the different results
        # for the two queries below.
        occurrences.search(datasetKey='7b5d6a48-f762-11e1-a439-00145eb45e9a', offset=10, limit=5)
        occurrences.search(datasetKey='7b5d6a48-f762-11e1-a439-00145eb45e9a', offset=20, limit=5)

        # Many dataset keys
        # occurrences.search(datasetKey=["50c9509d-22c7-4a22-a47d-8c48425ef4a7", "7b5d6a48-f762-11e1-a439-00145eb45e9a"], limit=20)

        # Search by collector name
        res = occurrences.search(recordedBy="smith", limit=20)
        [ x['recordedBy'] for x in res['results'] ]

        # Many collector names
        # occurrences.search(recordedBy=["smith","BJ Stacey"], limit=20)

        # Search for many species
        splist = ['Cyanocitta stelleri', 'Junco hyemalis', 'Aix sponsa']
        keys = [ species.name_suggest(x)[0]['key'] for x in splist ]
        out = [ occurrences.search(taxonKey = x, limit=1) for x in keys ]
        [ x['results'][0]['speciesKey'] for x in out ]

        # Search - q parameter
        occurrences.search(q="kingfisher", limit=2)

        # Search on latitidue and longitude
        occurrences.search(decimalLatitude=50, decimalLongitude=10, limit=2)

        # Search on a bounding box
        ## in well known text format
        occurrences.search(geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))', limit=20)
        from pygbif import species
        key = species.name_suggest(q='Aesculus hippocastanum')[0]['key']
        occurrences.search(taxonKey=key, geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))', limit=20)

        # Search on country
        occurrences.search(country='US', limit=20)
        occurrences.search(country='FR', limit=20)
        occurrences.search(country='DE', limit=20)

        # Get only occurrences with lat/long data
        occurrences.search(taxonKey=key, hasCoordinate=True, limit=20)

        # Get only occurrences that were recorded as living specimens
        occurrences.search(taxonKey=key, basisOfRecord="LIVING_SPECIMEN", hasCoordinate=True, limit=20)

        # Get occurrences for a particular eventDate
        occurrences.search(taxonKey=key, eventDate="2013", limit=20)
        occurrences.search(taxonKey=key, year="2013", limit=20)
        occurrences.search(taxonKey=key, month="6", limit=20)

        # Get occurrences based on depth
        key = species.name_backbone(name='Salmo salar', kingdom='animals')['usageKey']
        occurrences.search(taxonKey=key, depth="5", limit=20)

        # Get occurrences based on elevation
        key = species.name_backbone(name='Puma concolor', kingdom='animals')['usageKey']
        occurrences.search(taxonKey=key, elevation=50, hasCoordinate=True, limit=20)

        # Get occurrences based on institutionCode
        occurrences.search(institutionCode="TLMF", limit=20)

        # Get occurrences based on collectionCode
        occurrences.search(collectionCode="Floristic Databases MV - Higher Plants", limit=20)

        # Get only those occurrences with spatial issues
        occurrences.search(taxonKey=key, hasGeospatialIssue=True, limit=20)

        # Search using a query string
        occurrences.search(q="kingfisher", limit=20)

        # Range queries
        ## See Detail for parameters that support range queries
        ### this is a range depth, with lower/upper limits in character string
        occurrences.search(depth='50,100')

        ## Range search with year
        occurrences.search(year='1999,2000', limit=20)

        ## Range search with latitude
        occurrences.search(decimalLatitude='29.59,29.6')

        # Search by specimen type status
        ## Look for possible values of the typeStatus parameter looking at the typestatus dataset
        occurrences.search(typeStatus = 'allotype')

        # Search by specimen record number
        ## This is the record number of the person/group that submitted the data, not GBIF's numbers
        ## You can see that many different groups have record number 1, so not super helpful
        occurrences.search(recordNumber = 1)

        # Search by last time interpreted: Date the record was last modified in GBIF
        ## The lastInterpreted parameter accepts ISO 8601 format dates, including
        ## yyyy, yyyy-MM, yyyy-MM-dd, or MM-dd. Range queries are accepted for lastInterpreted
        occurrences.search(lastInterpreted = '2014-04-01')

        # Search by continent
        ## One of africa, antarctica, asia, europe, north_america, oceania, or south_america
        occurrences.search(continent = 'south_america')
        occurrences.search(continent = 'africa')
        occurrences.search(continent = 'oceania')
        occurrences.search(continent = 'antarctica')

        # Search for occurrences with images
        occurrences.search(mediatype = 'StillImage')
        occurrences.search(mediatype = 'MovingImage')
        x = occurrences.search(mediatype = 'Sound')
        [z['media'] for z in x['results']]

        # Query based on issues
        occurrences.search(taxonKey=1, issue='DEPTH_UNLIKELY')
        occurrences.search(taxonKey=1, issue=['DEPTH_UNLIKELY','COORDINATE_ROUNDED'])
        # Show all records in the Arizona State Lichen Collection that cant be matched to the GBIF
        # backbone properly:
        occurrences.search(datasetKey='84c0e1a0-f762-11e1-a439-00145eb45e9a', issue=['TAXON_MATCH_NONE','TAXON_MATCH_HIGHERRANK'])

        # If you pass in an invalid polygon you get hopefully informative errors
        ### the WKT string is fine, but GBIF says bad polygon
        wkt = 'POLYGON((-178.59375 64.83258989321493,-165.9375 59.24622380205539,
        -147.3046875 59.065977905449806,-130.78125 51.04484764446178,-125.859375 36.70806354647625,
        -112.1484375 23.367471303759686,-105.1171875 16.093320185359257,-86.8359375 9.23767076398516,
        -82.96875 2.9485268155066175,-82.6171875 -14.812060061226388,-74.8828125 -18.849111862023985,
        -77.34375 -47.661687803329166,-84.375 -49.975955187343295,174.7265625 -50.649460483096114,
        179.296875 -42.19189902447192,-176.8359375 -35.634976650677295,176.8359375 -31.835565983656227,
        163.4765625 -6.528187613695323,152.578125 1.894796132058301,135.703125 4.702353722559447,
        127.96875 15.077427674847987,127.96875 23.689804541429606,139.921875 32.06861069132688,
        149.4140625 42.65416193033991,159.2578125 48.3160811030533,168.3984375 57.019804336633165,
        178.2421875 59.95776046458139,-179.6484375 61.16708631440347,-178.59375 64.83258989321493))'
        occurrences.search(geometry = wkt)
    '''
    url = gbif_baseurl + 'occurrence/search'
    out = gbif_GET(url, {'taxonKey': taxonKey, 'scientificName': scientificName,
        'country': country, 'publishingCountry': publishingCountry,
        'hasCoordinate': hasCoordinate, 'typeStatus': typeStatus,
        'recordNumber': recordNumber, 'lastInterpreted': lastInterpreted,
        'continent': continent, 'geometry': geometry, 'recordedBy': recordedBy,
        'basisOfRecord': basisOfRecord, 'datasetKey': datasetKey, 'eventDate': eventDate,
        'catalogNumber': catalogNumber, 'year': year, 'month': month,
        'decimalLatitude': decimalLatitude, 'decimalLongitude': decimalLongitude,
        'elevation': elevation, 'depth': depth, 'institutionCode': institutionCode,
        'collectionCode': collectionCode, 'hasGeospatialIssue': hasGeospatialIssue,
        'issue': issue, 'q': q, 'mediatype': mediatype, 'limit': limit,
        'offset': offset}, **kwargs)
    return out



from ..gbifutils import *

def count(taxonKey=None, basisOfRecord=None, country=None, isGeoreferenced=None,
    datasetKey=None, publishingCountry=None, typeStatus=None,
    issue=None, year=None, **kwargs):
    '''
    Returns occurrence counts for a predefined set of dimensions

    :param taxonKey: [int] A GBIF occurrence identifier
    :param basisOfRecord: [str] A GBIF occurrence identifier
    :param country: [str] A GBIF occurrence identifier
    :param isGeoreferenced: [bool] A GBIF occurrence identifier
    :param datasetKey: [str] A GBIF occurrence identifier
    :param publishingCountry: [str] A GBIF occurrence identifier
    :param typeStatus: [str] A GBIF occurrence identifier
    :param issue: [str] A GBIF occurrence identifier
    :param year: [int] A GBIF occurrence identifier

    :return: dict

    Usage::

        from pygbif import occurrences
        occurrences.count(taxonKey = 3329049)
        occurrences.count(country = 'CA')
        occurrences.count(isGeoreferenced = True)
        occurrences.count(basisOfRecord = 'OBSERVATION')
    '''
    url = gbif_baseurl + 'occurrence/count'
    out = gbif_GET(url, {'taxonKey': taxonKey, 'basisOfRecord': basisOfRecord, 'country': country,
        'isGeoreferenced': isGeoreferenced, 'datasetKey': datasetKey,
        'publishingCountry': publishingCountry, 'typeStatus': typeStatus,
        'issue': issue, 'year': year}, **kwargs)
    return out

def count_basisofrecord(**kwargs):
    '''
    Lists occurrence counts by basis of record.

    :return: dict

    Usage::

            from pygbif import occurrences
            occurrences.count_basisofrecord()
    '''
    url = gbif_baseurl + 'occurrence/counts/basisOfRecord'
    out = gbif_GET(url, {}, **kwargs)
    return out

def count_year(year, **kwargs):
    '''
    Lists occurrence counts by year

    :param year: [int] year range, e.g., "1990,2000". Does not support ranges like "asterisk,2010"

    :return: dict

    Usage::

            from pygbif import occurrences
            occurrences.count_year(year = '1990,2000')
    '''
    url = gbif_baseurl + 'occurrence/counts/year'
    out = gbif_GET(url, {'year': year}, **kwargs)
    return out

def count_datasets(taxonKey = None, country = None, **kwargs):
    '''
    Lists occurrence counts for datasets that cover a given taxon or country

    :param taxonKey: [int] Taxon key
    :param country: [str] A country, two letter code

    :return: dict

    Usage::

            from pygbif import occurrences
            occurrences.count_datasets(country = "DE")
    '''
    url = gbif_baseurl + 'occurrence/counts/datasets'
    out = gbif_GET(url, {'taxonKey': taxonKey, 'country': country}, **kwargs)
    return out

def count_countries(publishingCountry, **kwargs):
    '''
    Lists occurrence counts for all countries covered by the data published by the given country

    :param publishingCountry: [str] A two letter country code

    :return: dict

    Usage::

            from pygbif import occurrences
            occurrences.count_countries(publishingCountry = "DE")
    '''
    url = gbif_baseurl + 'occurrence/counts/countries'
    out = gbif_GET(url, {'publishingCountry': publishingCountry}, **kwargs)
    return out

def count_publishingcountries(country, **kwargs):
    '''
    Lists occurrence counts for all countries that publish data about the given country

    :param country: [str] A country, two letter code

    :return: dict

    Usage::

            from pygbif import occurrences
            occurrences.count_publishingcountries(country = "DE")
    '''
    url = gbif_baseurl + 'occurrence/counts/publishingCountries'
    out = gbif_GET(url, {"country": country}, **kwargs)
    return out

def count_schema(**kwargs):
    '''
    List the supported metrics by the service

    :return: dict

    Usage::

            from pygbif import occurrences
            occurrences.count_schema()
    '''
    url = gbif_baseurl + 'occurrence/count/schema'
    out = gbif_GET(url, {}, **kwargs)
    return out



from ..gbifutils import *
from .registry_utils import *

def installations(data = 'all', uuid = None, query = None, identifier = None,
  identifierType = None, limit = 100, start = None, **kwargs):
  '''
  Installations metadata.

  :param data: The type of data to get. Default is all data. If not 'all', then one
     or more of 'contact', 'endpoint', 'dataset', 'comment', 'deleted', 'nonPublishing'.
  :param uuid: UUID of the data node provider. This must be specified if data
     is anything other than 'all'.
  :param query: Query nodes. Only used when data='all'. Ignored otherwise.

  References: http://www.gbif.org/developer/registry#installations

  Usage::

      from pygbif import registry
      registry.installations(limit=5)
      registry.installations(query="france")
      registry.installations(uuid="b77901f9-d9b0-47fa-94e0-dd96450aa2b4")
      registry.installations(data='contact', uuid="b77901f9-d9b0-47fa-94e0-dd96450aa2b4")
      registry.installations(data='contact', uuid="2e029a0c-87af-42e6-87d7-f38a50b78201")
      registry.installations(data='endpoint', uuid="b77901f9-d9b0-47fa-94e0-dd96450aa2b4")
      registry.installations(data='dataset', uuid="b77901f9-d9b0-47fa-94e0-dd96450aa2b4")
      registry.installations(data='deleted')
      registry.installations(data='deleted', limit=2)
      registry.installations(data=['deleted','nonPublishing'], limit=2)
      registry.installations(identifierType='DOI', limit=2)
  '''
  args = {'q': query, 'limit': limit, 'offset': start}
  data_choices = ['all', 'contact', 'endpoint', 'dataset',
   'identifier', 'tag', 'machineTag', 'comment',
   'deleted', 'nonPublishing']
  check_data(data, data_choices)

  def getdata(x, uuid, args, **kwargs):
    if x not in ['all','deleted', 'nonPublishing'] and uuid is None:
      stop('You must specify a uuid if data does not equal all and data does not equal one of deleted or nonPublishing')

    if uuid is None:
      if x is 'all':
        url = gbif_baseurl + 'installation'
      else:
        url = gbif_baseurl + 'installation/' + x
    else:
      if x is 'all':
        url = gbif_baseurl + 'installation/' + uuid
      else:
        url = gbif_baseurl + 'installation/' + uuid + '/' + x

    res = gbif_GET(url, args, **kwargs)
    return {'meta': get_meta(res), 'data': parse_results(res, uuid)}

  if len2(data) == 1:
    return getdata(data, uuid, args, **kwargs)
  else:
    return [getdata(x, uuid, args, **kwargs) for x in data]



from ..gbifutils import *

def get(key, **kwargs):
    '''
    Gets details for a single, interpreted occurrence

    :param key: [Fixnum] A GBIF occurrence key

    :return: A dictionary, of results

    Usage::

        from pygbif import occurrences
        occurrences.get(key = 252408386)
    '''
    url = gbif_baseurl + 'occurrence/' + str(key)
    out = gbif_GET(url, {}, **kwargs)
    return out

def get_verbatim(key, **kwargs):
    '''
    Gets a verbatim occurrence record without any interpretation

    :param key: [Fixnum] A GBIF occurrence key

    :return: A dictionary, of results

    Usage::

        from pygbif import occurrences
        occurrences.get_verbatim(key = 252408386)
    '''
    url = gbif_baseurl + 'occurrence/' + str(key) + '/verbatim'
    out = gbif_GET(url, {}, **kwargs)
    return out

def get_fragment(key, **kwargs):
    '''
    Get a single occurrence fragment in its raw form (xml or json)

    :param key: [Fixnum] A GBIF occurrence key

    :return: A dictionary, of results

    Usage::

        from pygbif import occurrences
        occurrences.get_fragment(key = 1052909293)
    '''
    url = gbif_baseurl + 'occurrence/' + str(key) + '/fragment'
    out = gbif_GET(url, {}, **kwargs)
    return out
